"""
Module providing helpers to manipulate jobs
"""

import os
import shutil
import quixote
from quixote.build import GeneratorType
import panza.build
from panza.config import get_config


class ExecutableJobHandle:
    """
    Class representing a job that is ready to be executed
    """

    def __init__(self, root_dir: str, job_info):
        self.root_dir = root_dir
        self.job_info = job_info

    def run(self):
        """
        Execute the job
        """
        image_name = f"test-{self.job_info['module_id']}-{self.job_info['activity_id']}"
        os.system(f"docker run -ti -v {self.root_dir}:/moulinette {image_name} panza_executor.py")
        os.system(f"rm -rf {self.root_dir}")


class DataAwaitingJobHandle:
    """
    Class representing a job whose environment is already built, and needs to fetch data to operate on
    """

    def __init__(self, root_dir: str, job_info, blueprint: quixote.Blueprint):
        self.root_dir = root_dir
        self.job_info = job_info
        self.blueprint = blueprint

    def fetch_data(self) -> ExecutableJobHandle:
        """
        Fetch the data on which this job will operate

        The fetcher functions specified in the blueprint will be used
        """
        delivery_path = os.path.join(self.root_dir, "rendu")
        resources_path = os.path.join(self.root_dir, "resources")
        with quixote.new_context(
                **get_config(),
                **self.job_info,
                delivery_path=delivery_path,
                resources_path=resources_path
        ):
            for fetcher in self.blueprint.fetchers:
                fetcher()
        return ExecutableJobHandle(self.root_dir, self.job_info)


class JobWorkspaceHandle:
    """
    Class representing a job workspace, that is, a job with its working directory and its resources
    """

    def __init__(self, root_dir: str, job_info, blueprint: quixote.Blueprint):
        self.root_dir = root_dir
        self.job_info = job_info
        self.blueprint = blueprint

    def build_job_environment(self) -> DataAwaitingJobHandle:
        """
        Build the environment specified by the blueprint for this job.

        The builder functions specified in the blueprint will be used to build a suitable Docker image
        """
        env_dir = os.path.join(self.root_dir, "env")
        os.mkdir(env_dir)
        generator = panza.build.create_script_generator(GeneratorType.DOCKER)
        generator.generate_to_file(self.blueprint, os.path.join(env_dir, "Dockerfile"), extra_context=get_config())
        os.system(f"docker build -t test-{self.job_info['module_id']}-{self.job_info['activity_id']} {env_dir}")
        shutil.rmtree(env_dir)
        return DataAwaitingJobHandle(self.root_dir, self.job_info, self.blueprint)


class JobInfo:
    """
    Class representing a job information, that is, a directory containing the job's resources, and its metadata
    """

    def __init__(self, job_dir: str, job_info):
        self.job_dir = job_dir
        self.job_info = job_info

    def create_workspace(self) -> JobWorkspaceHandle:
        """
        Create a workspace in which the job can be executed.

        This will create a directory, copy the required files, and load the blueprint
        """
        job_name = f"{self.job_info['module_id']}-{self.job_info['activity_id']}-{self.job_info['group_id']}"
        root_dir = os.path.join(get_config()["root_dir"], job_name)
        shutil.copytree(self.job_dir, root_dir)

        blueprint = panza.BlueprintLoader.load_from_directory(root_dir)

        return JobWorkspaceHandle(root_dir, self.job_info, blueprint)
